@@ -11,20 +11,20 @@ from bokeh.models import ( ColumnDataSource, Plot, DataRange1d, LinearAxis,
         Quad, TapTool, OpenURL, Button, Select)
 from bokeh.palettes import Spectral9
 from bokeh.plotting import figure
+import dask
 from tornado import gen
 
-from ..config import config
 from ..diagnostics.progress_stream import nbytes_bar
 from .. import profile
 from ..utils import log_errors, parse_timedelta
 
-if config.get('bokeh-export-tool', False):
+if dask.config.get('distributed.dashboard.export-tool'):
     from .export_tool import ExportTool
 else:
     ExportTool = None
 
 
-profile_interval = config.get('profile-interval', 10)
+profile_interval = dask.config.get('distributed.worker.profile.interval')
 profile_interval = parse_timedelta(profile_interval, default='ms')
 
 
@@ -373,7 +373,11 @@ class ProfileTimePlot(DashboardComponent):
         self.states = data.pop('states')
         self.source = ColumnDataSource(data=data)
 
+        changing = [False]  # avoid repeated changes from within callback
+
         def cb(attr, old, new):
+            if changing[0]:
+                return
             with log_errors():
                 try:
                     ind = new['1d']['indices'][0]
@@ -382,14 +386,18 @@ class ProfileTimePlot(DashboardComponent):
                 data = profile.plot_data(self.states[ind], profile_interval)
                 del self.states[:]
                 self.states.extend(data.pop('states'))
+                changing[0] = True  # don't recursively trigger callback
                 self.source.data.update(data)
                 self.source.selected = old
+                changing[0] = False
 
         self.source.on_change('selected', cb)
 
         self.profile_plot = figure(tools='tap', height=400, **kwargs)
-        self.profile_plot.quad('left', 'right', 'top', 'bottom', color='color',
-                               line_color='black', source=self.source)
+        r = self.profile_plot.quad('left', 'right', 'top', 'bottom', color='color',
+                                   line_color='black', source=self.source)
+        r.selection_glyph = None
+        r.nonselection_glyph = None
 
         hover = HoverTool(
             point_policy="follow_mouse",
