@@ -11,13 +11,13 @@ try:
 except ImportError:
     ssl = None
 
+import dask
 import tornado
 from tornado import gen, netutil
 from tornado.iostream import StreamClosedError, IOStream
 from tornado.tcpclient import TCPClient
 from tornado.tcpserver import TCPServer
 
-from .. import config
 from ..compatibility import finalize, PY3
 from ..utils import (ensure_bytes, ensure_ip, get_ip, get_ipv6, nbytes,
                      parse_timedelta, shutting_down)
@@ -42,8 +42,6 @@ def get_total_physical_memory():
 
 MAX_BUFFER_SIZE = get_total_physical_memory()
 
-DEFAULT_BACKLOG = 2048
-
 
 def set_tcp_timeout(stream):
     """
@@ -52,7 +50,7 @@ def set_tcp_timeout(stream):
     if stream.closed():
         return
 
-    timeout = config.get('tcp-timeout', 30)
+    timeout = dask.config.get('distributed.comm.timeouts.tcp')
     timeout = int(parse_timedelta(timeout, default='seconds'))
 
     sock = stream.socket
@@ -170,7 +168,7 @@ class TCP(Comm):
         return self._peer_addr
 
     @gen.coroutine
-    def read(self):
+    def read(self, deserializers=None):
         stream = self.stream
         if stream is None:
             raise CommClosedError
@@ -198,23 +196,29 @@ class TCP(Comm):
             self.stream = None
             if not shutting_down():
                 convert_stream_closed_error(self, e)
-
-        try:
-            msg = yield from_frames(frames, deserialize=self.deserialize)
-        except EOFError:
-            # Frames possibly garbled or truncated by communication error
-            self.abort()
-            raise CommClosedError("aborted stream on truncated data")
-        raise gen.Return(msg)
+        else:
+            try:
+                msg = yield from_frames(frames,
+                                        deserialize=self.deserialize,
+                                        deserializers=deserializers)
+            except EOFError:
+                # Frames possibly garbled or truncated by communication error
+                self.abort()
+                raise CommClosedError("aborted stream on truncated data")
+            raise gen.Return(msg)
 
     @gen.coroutine
-    def write(self, msg):
+    def write(self, msg, serializers=None, on_error='message'):
         stream = self.stream
         bytes_since_last_yield = 0
         if stream is None:
             raise CommClosedError
 
-        frames = yield to_frames(msg)
+        frames = yield to_frames(msg,
+                                 serializers=serializers,
+                                 on_error=on_error,
+                                 context={'sender': self._local_addr,
+                                          'recipient': self._peer_addr})
 
         try:
             lengths = ([struct.pack('Q', len(frames))] +
@@ -365,7 +369,7 @@ class BaseTCPListener(Listener, RequireEncryptionMixin):
         self.tcp_server = TCPServer(max_buffer_size=MAX_BUFFER_SIZE,
                                     **self.server_args)
         self.tcp_server.handle_stream = self._handle_stream
-        backlog = int(config.get('socket-backlog', DEFAULT_BACKLOG))
+        backlog = int(dask.config.get('distributed.comm.socket-backlog'))
         for i in range(5):
             try:
                 # When shuffling data between workers, there can
