@@ -3,7 +3,6 @@ from __future__ import print_function, division, absolute_import
 from collections import defaultdict, deque
 from concurrent.futures import CancelledError
 from functools import partial
-import inspect
 import logging
 import six
 import traceback
@@ -17,14 +16,14 @@ from tornado import gen
 from tornado.ioloop import IOLoop
 from tornado.locks import Event
 
-from .compatibility import PY3, get_thread_identity
+from .compatibility import get_thread_identity
 from .comm import (connect, listen, CommClosedError,
                    normalize_address,
                    unparse_host_port, get_address_host_port)
 from .metrics import time
 from .system_monitor import SystemMonitor
 from .utils import (get_traceback, truncate_exception, ignoring, shutting_down,
-                    PeriodicCallback, parse_timedelta)
+                    PeriodicCallback, parse_timedelta, has_keyword)
 from . import protocol
 
 
@@ -310,7 +309,7 @@ class Server(object):
                     logger.warning("No handler %s found in %s", op,
                                    type(self).__name__, exc_info=True)
                 else:
-                    if serializers is not None and has_serializers_keyword(handler):
+                    if serializers is not None and has_keyword(handler, 'serializers'):
                         msg['serializers'] = serializers  # add back in
 
                     logger.debug("Calling into handler %s", handler.__name__)
@@ -409,8 +408,8 @@ def pingpong(comm):
 
 
 @gen.coroutine
-def send_recv(comm, reply=True, deserialize=True, serializers=None,
-              deserializers=None, **kwargs):
+def send_recv(comm, reply=True, serializers=None, deserializers=None,
+              **kwargs):
     """ Send and recv with a Comm.
 
     Keyword arguments turn into the message
@@ -443,7 +442,10 @@ def send_recv(comm, reply=True, deserialize=True, serializers=None,
             comm.abort()
 
     if isinstance(response, dict) and response.get('status') == 'uncaught-error':
-        six.reraise(*clean_exception(**response))
+        if comm.deserialize:
+            six.reraise(*clean_exception(**response))
+        else:
+            raise Exception(response['text'])
     raise gen.Return(response)
 
 
@@ -598,6 +600,10 @@ class PooledRPCCall(object):
         self.serializers = serializers
         self.deserializers = deserializers if deserializers is not None else serializers
 
+    @property
+    def address(self):
+        return self.addr
+
     def __getattr__(self, key):
         @gen.coroutine
         def send_recv_from_rpc(**kwargs):
@@ -831,7 +837,10 @@ def error_message(e, status='error'):
     else:
         tb_result = protocol.to_serialize(tb)
 
-    return {'status': status, 'exception': e4, 'traceback': tb_result}
+    return {'status': status,
+            'exception': e4,
+            'traceback': tb_result,
+            'text': str(e2)}
 
 
 def clean_exception(exception, traceback, **kwargs):
@@ -848,13 +857,3 @@ def clean_exception(exception, traceback, **kwargs):
     elif isinstance(traceback, string_types):
         traceback = None  # happens if the traceback failed serializing
     return type(exception), exception, traceback
-
-
-def has_serializers_keyword(func):
-    if PY3:
-        return 'serializers' in inspect.signature(func).parameters
-    else:
-        # https://stackoverflow.com/questions/50100498/determine-keywords-of-a-tornado-coroutine
-        if gen.is_coroutine_function(func):
-            func = func.__wrapped__
-        return 'serializers' in inspect.getargspec(func).args
