@@ -17,7 +17,9 @@ from distributed import Client, Worker, Nanny
 from distributed.deploy.local import LocalCluster
 from distributed.metrics import time
 from distributed.utils_test import (inc, gen_test, slowinc,
+                                    assert_cannot_connect,
                                     assert_can_connect_locally_4,
+                                    assert_can_connect_from_everywhere_4,
                                     assert_can_connect_from_everywhere_4_6,
                                     captured_logger)
 from distributed.utils_test import loop  # noqa: F401
@@ -37,6 +39,7 @@ def test_simple(loop):
             assert any(w.data == {x.key: 2} for w in c.workers)
 
 
+@pytest.mark.skipif('sys.version_info[0] == 2', reason='fork issues')
 def test_close_twice():
     with LocalCluster() as cluster:
         with Client(cluster.scheduler_address) as client:
@@ -124,6 +127,7 @@ class LocalTest(ClusterTest, unittest.TestCase):
     Cluster = partial(LocalCluster, silence_logs=False, diagnostics_port=None)
 
 
+@pytest.mark.skipif('sys.version_info[0] == 2', reason='')
 def test_Client_with_local(loop):
     with LocalCluster(1, scheduler_port=0, silence_logs=False,
                       diagnostics_port=None, loop=loop) as c:
@@ -151,6 +155,7 @@ def test_Client_twice(loop):
             assert c.cluster.scheduler.port != f.cluster.scheduler.port
 
 
+@pytest.mark.skipif('sys.version_info[0] == 2', reason='fork issues')
 def test_defaults():
     from distributed.worker import _ncores
 
@@ -244,6 +249,7 @@ def test_bokeh(loop, processes):
         requests.get(url, timeout=0.2)
 
 
+@pytest.mark.skipif(sys.version_info < (3, 6), reason='Unknown')
 def test_blocks_until_full(loop):
     with Client(loop=loop) as c:
         assert len(c.ncores()) > 0
@@ -444,5 +450,54 @@ def test_adapt_then_manual(loop):
                 assert time() < start + 5
 
 
+def test_local_tls(loop):
+    from distributed.utils_test import tls_only_security
+    security = tls_only_security()
+    with LocalCluster(scheduler_port=8786, silence_logs=False, security=security,
+                      diagnostics_port=0, ip='tls://0.0.0.0', loop=loop) as c:
+        sync(loop, assert_can_connect_from_everywhere_4, c.scheduler.port,
+             connection_args=security.get_connection_args('client'),
+             protocol='tls', timeout=3)
+
+        # If we connect to a TLS localculster without ssl information we should fail
+        sync(loop, assert_cannot_connect,
+             addr='tcp://127.0.0.1:%d' % c.scheduler.port,
+             connection_args=security.get_connection_args('client'),
+             exception_class=RuntimeError,
+             )
+
+
+@gen_test()
+def test_scale_retires_workers():
+    class MyCluster(LocalCluster):
+        def scale_down(self, *args, **kwargs):
+            pass
+
+    loop = IOLoop.current()
+    cluster = yield MyCluster(0, scheduler_port=0, processes=False,
+                              silence_logs=False, diagnostics_port=None,
+                              loop=loop, asynchronous=True)
+    c = yield Client(cluster, loop=loop, asynchronous=True)
+
+    assert not cluster.workers
+
+    yield cluster.scale(2)
+
+    start = time()
+    while len(cluster.scheduler.workers) != 2:
+        yield gen.sleep(0.01)
+        assert time() < start + 3
+
+    yield cluster.scale(1)
+
+    start = time()
+    while len(cluster.scheduler.workers) != 1:
+        yield gen.sleep(0.01)
+        assert time() < start + 3
+
+    yield c._close()
+    yield cluster._close()
+
+
 if sys.version_info >= (3, 5):
     from distributed.deploy.tests.py3_test_deploy import *  # noqa F401
