@@ -1,6 +1,6 @@
 from __future__ import print_function, division, absolute_import
-
 from functools import partial
+import traceback
 
 from dask.base import normalize_token
 try:
@@ -12,92 +12,95 @@ import msgpack
 
 from . import pickle
 from ..compatibility import PY2
+from ..utils import has_keyword
 from .compression import maybe_compress, decompress
 from .utils import unpack_frames, pack_frames_prelude, frame_split_size
 
 
-serializers = {}
-deserializers = {None: lambda header, frames: pickle.loads(b''.join(frames))}
+class_serializers = {}
 
 lazy_registrations = {}
 
 
-def register_serialization(cls, serialize, deserialize):
-    """ Register a new class for custom serialization
-
-    Parameters
-    ----------
-    cls: type
-    serialize: function
-    deserialize: function
-
-    Examples
-    --------
-    >>> class Human(object):
-    ...     def __init__(self, name):
-    ...         self.name = name
-
-    >>> def serialize(human):
-    ...     header = {}
-    ...     frames = [human.name.encode()]
-    ...     return header, frames
-
-    >>> def deserialize(header, frames):
-    ...     return Human(frames[0].decode())
-
-    >>> register_serialization(Human, serialize, deserialize)
-    >>> serialize(Human('Alice'))
-    ({}, [b'Alice'])
-
-    See Also
-    --------
-    serialize
-    deserialize
-    """
-    if isinstance(cls, type):
-        name = typename(cls)
-    elif isinstance(cls, str):
-        name = cls
-    serializers[name] = serialize
-    deserializers[name] = deserialize
-
-
-def register_serialization_lazy(toplevel, func):
-    """Register a registration function to be called if *toplevel*
-    module is ever loaded.
-    """
-    lazy_registrations[toplevel] = func
-
-
-def typename(typ):
-    """ Return name of type
-
-    Examples
-    --------
-    >>> from distributed import Scheduler
-    >>> typename(Scheduler)
-    'distributed.scheduler.Scheduler'
-    """
-    return typ.__module__ + '.' + typ.__name__
-
-
-def _find_lazy_registration(typename):
-    toplevel, _, _ = typename.partition('.')
-    if toplevel in lazy_registrations:
-        lazy_registrations.pop(toplevel)()
-        return True
+def dask_dumps(x, context=None):
+    """Serialise object using the class-based registry"""
+    typ = typename(type(x))
+    if typ in class_serializers:
+        dumps, loads, has_context = class_serializers[typ]
+        if has_context:
+            header, frames = dumps(x, context=context)
+        else:
+            header, frames = dumps(x)
+        header['type'] = typ
+        header['serializer'] = 'dask'
+        return header, frames
+    elif _find_lazy_registration(typ):
+        return dask_dumps(x)  # recurse
     else:
-        return False
+        raise NotImplementedError(typ)
 
 
-def serialize(x):
+def dask_loads(header, frames):
+    typ = header['type']
+
+    if typ not in class_serializers:
+        _find_lazy_registration(typ)
+
+    try:
+        dumps, loads, _ = class_serializers[typ]
+    except KeyError:
+        raise TypeError("Serialization for type %s not found" % typ)
+    else:
+        return loads(header, frames)
+
+
+def pickle_dumps(x):
+    return {'serializer': 'pickle'}, [pickle.dumps(x)]
+
+
+def pickle_loads(header, frames):
+    return pickle.loads(b''.join(frames))
+
+
+def msgpack_dumps(x):
+    return {'serializer': 'msgpack'}, [msgpack.dumps(x, use_bin_type=True)]
+
+
+def msgpack_loads(header, frames):
+    return msgpack.loads(b''.join(frames), encoding='utf8', use_list=False)
+
+
+def serialization_error_loads(header, frames):
+    msg = '\n'.join([frame.decode('utf8') for frame in frames])
+    raise TypeError(msg)
+
+
+families = {}
+
+
+def register_serialization_family(name, dumps, loads):
+    families[name] = (dumps, loads, dumps and has_keyword(dumps, 'context'))
+
+
+register_serialization_family('dask', dask_dumps, dask_loads)
+register_serialization_family('pickle', pickle_dumps, pickle_loads)
+register_serialization_family('msgpack', msgpack_dumps, msgpack_loads)
+register_serialization_family('error', None, serialization_error_loads)
+
+
+def serialize(x, serializers=None, on_error='message', context=None):
     r"""
     Convert object to a header and list of bytestrings
 
     This takes in an arbitrary Python object and returns a msgpack serializable
-    header and a list of bytes or memoryview objects.  By default this uses
-    pickle/cloudpickle but can use special functions if they have been
-    pre-registered.
+    header and a list of bytes or memoryview objects.
+
+    The serialization protocols to use are configurable: a list of names
+    define the set of serializers to use, in order. These names are keys in
+    the ``serializer_registry`` dict (e.g., 'pickle', 'msgpack'), which maps
+    to the de/serialize functions. The name 'dask' is special, and will use the
+    per-class serialization methods. ``None`` gives the default list
+    ``['dask', 'pickle']``.
 
     Examples
     --------
@@ -121,23 +124,40 @@ def serialize(x):
     to_serialize: Mark that data in a message should be serialized
     register_serialization: Register custom serialization functions
     """
+    if serializers is None:
+        serializers = ('dask', 'pickle')  # TODO: get from configuration
+
     if isinstance(x, Serialized):
         return x.header, x.frames
 
-    typ = type(x)
-    name = typename(typ)
-    if name in serializers:
-        header, frames = serializers[name](x)
-        header['type'] = name
-    else:
-        if _find_lazy_registration(name):
-            return serialize(x)  # recurse
-        header, frames = {}, [pickle.dumps(x)]
+    tb = ''
 
-    return header, frames
+    for name in serializers:
+        dumps, loads, wants_context = families[name]
+        try:
+            header, frames = dumps(x, context=context) if wants_context else dumps(x)
+            header['serializer'] = name
+            return header, frames
+        except NotImplementedError:
+            continue
+        except Exception:
+            tb = traceback.format_exc()
+            continue
+
+    msg = "Could not serialize object of type %s" % type(x).__name__
+    if on_error == 'message':
+        frames = [msg]
+        if tb:
+            frames.append(tb[:100000])
+
+        frames = [frame.encode() for frame in frames]
+
+        return {'serializer': 'error'}, frames
+    elif on_error == 'raise':
+        raise TypeError(msg)
 
 
-def deserialize(header, frames):
+def deserialize(header, frames, deserializers=None):
     """
     Convert serialized header and list of bytestrings back to a Python object
 
@@ -150,12 +170,12 @@ def deserialize(header, frames):
     --------
     serialize
     """
-    name = header.get('type')
-    if name not in deserializers:
-        if _find_lazy_registration(name):
-            return deserialize(header, frames)  # recurse
-    f = deserializers[header.get('type')]
-    return f(header, frames)
+    name = header.get('serializer')
+    if deserializers is not None and name not in deserializers:
+        raise TypeError("Data serialized with %s but only able to deserialize "
+                        "data with %s" % (name, str(list(deserializers))))
+    dumps, loads, wants_context = families[name]
+    return loads(header, frames)
 
 
 class Serialize(object):
@@ -316,6 +336,113 @@ def nested_deserialize(x):
     return replace_inner(x)
 
 
+def serialize_bytelist(x, **kwargs):
+    header, frames = serialize(x, **kwargs)
+    frames = frame_split_size(frames)
+    if frames:
+        compression, frames = zip(*map(maybe_compress, frames))
+    else:
+        compression = []
+    header['compression'] = compression
+    header['count'] = len(frames)
+
+    header = msgpack.dumps(header, use_bin_type=True)
+    frames2 = [header] + list(frames)
+    return [pack_frames_prelude(frames2)] + frames2
+
+
+def serialize_bytes(x, **kwargs):
+    L = serialize_bytelist(x, **kwargs)
+    if PY2:
+        L = [bytes(y) for y in L]
+    return b''.join(L)
+
+
+def deserialize_bytes(b):
+    frames = unpack_frames(b)
+    header, frames = frames[0], frames[1:]
+    if header:
+        header = msgpack.loads(header, encoding='utf8', use_list=False)
+    else:
+        header = {}
+    frames = decompress(header, frames)
+    return deserialize(header, frames)
+
+
+################################
+# Class specific serialization #
+################################
+
+
+def register_serialization(cls, serialize, deserialize):
+    """ Register a new class for dask-custom serialization
+
+    Parameters
+    ----------
+    cls: type
+    serialize: callable(cls) -> Tuple[Dict, List[bytes]]
+    deserialize: callable(header: Dict, frames: List[bytes]) -> cls
+
+    Examples
+    --------
+    >>> class Human(object):
+    ...     def __init__(self, name):
+    ...         self.name = name
+
+    >>> def serialize(human):
+    ...     header = {}
+    ...     frames = [human.name.encode()]
+    ...     return header, frames
+
+    >>> def deserialize(header, frames):
+    ...     return Human(frames[0].decode())
+
+    >>> register_serialization(Human, serialize, deserialize)
+    >>> serialize(Human('Alice'))
+    ({}, [b'Alice'])
+
+    See Also
+    --------
+    serialize
+    deserialize
+    """
+    if isinstance(cls, type):
+        name = typename(cls)
+    elif isinstance(cls, str):
+        name = cls
+    class_serializers[name] = (serialize,
+                               deserialize,
+                               has_keyword(serialize, 'context'))
+
+
+def register_serialization_lazy(toplevel, func):
+    """Register a registration function to be called if *toplevel*
+    module is ever loaded.
+    """
+    lazy_registrations[toplevel] = func
+
+
+def typename(typ):
+    """ Return name of type
+
+    Examples
+    --------
+    >>> from distributed import Scheduler
+    >>> typename(Scheduler)
+    'distributed.scheduler.Scheduler'
+    """
+    return typ.__module__ + '.' + typ.__name__
+
+
+def _find_lazy_registration(typename):
+    toplevel, _, _ = typename.partition('.')
+    if toplevel in lazy_registrations:
+        lazy_registrations.pop(toplevel)()
+        return True
+    else:
+        return False
+
+
 @partial(normalize_token.register, Serialized)
 def normalize_Serialized(o):
     return [o.header] + o.frames  # for dask.base.tokenize
@@ -337,36 +464,3 @@ def _deserialize_bytes(header, frames):
 # in practice.
 register_serialization(bytes, _serialize_bytes, _deserialize_bytes)
 register_serialization(bytearray, _serialize_bytes, _deserialize_bytes)
-
-
-def serialize_bytelist(x):
-    header, frames = serialize(x)
-    frames = frame_split_size(frames)
-    if frames:
-        compression, frames = zip(*map(maybe_compress, frames))
-    else:
-        compression = []
-    header['compression'] = compression
-    header['count'] = len(frames)
-
-    header = msgpack.dumps(header, use_bin_type=True)
-    frames2 = [header] + list(frames)
-    return [pack_frames_prelude(frames2)] + frames2
-
-
-def serialize_bytes(x):
-    L = serialize_bytelist(x)
-    if PY2:
-        L = [bytes(y) for y in L]
-    return b''.join(L)
-
-
-def deserialize_bytes(b):
-    frames = unpack_frames(b)
-    header, frames = frames[0], frames[1:]
-    if header:
-        header = msgpack.loads(header, encoding='utf8')
-    else:
-        header = {}
-    frames = decompress(header, frames)
-    return deserialize(header, frames)
