@@ -1,10 +1,12 @@
 from __future__ import print_function, division, absolute_import
 
 import atexit
-from collections import Iterable, deque
+from collections import deque
 from contextlib import contextmanager
 from datetime import timedelta
 import functools
+from hashlib import md5
+import inspect
 import json
 import logging
 import multiprocessing
@@ -32,14 +34,18 @@ try:
 except ImportError:
     resource = None
 
+import dask
 from dask import istask
-from toolz import memoize, valmap
+import toolz
 import tornado
 from tornado import gen
-from tornado.ioloop import IOLoop, PollIOLoop
+from tornado.ioloop import IOLoop
+try:
+    from tornado.ioloop import PollIOLoop
+except ImportError:
+    PollIOLoop = None  # dropped in tornado 6.0
 
 from .compatibility import Queue, PY3, PY2, get_thread_identity, unicode
-from .config import config
 from .metrics import time
 
 
@@ -56,7 +62,7 @@ no_default = '__no_default__'
 
 def _initialize_mp_context():
     if PY3 and not sys.platform.startswith('win') and 'PyPy' not in sys.version:
-        method = config.get('multiprocessing-method', 'forkserver')
+        method = dask.config.get('distributed.worker.multiprocessing-method')
         ctx = multiprocessing.get_context(method)
         # Makes the test suite much faster
         preload = ['distributed']
@@ -112,7 +118,7 @@ def get_fileno_limit():
         return 512
 
 
-@memoize
+@toolz.memoize
 def _get_ip(host, port, family, default):
     # By using a UDP socket, we don't actually try to connect but
     # simply select the local address through which *host* is reachable.
@@ -188,19 +194,41 @@ def ignore_exceptions(coroutines, *exceptions):
 
 
 @gen.coroutine
-def All(*args):
+def All(args, quiet_exceptions=()):
     """ Wait on many tasks at the same time
 
     Err once any of the tasks err.
 
     See https://github.com/tornadoweb/tornado/issues/1546
+
+    Parameters
+    ----------
+    args: futures to wait for
+    quiet_exceptions: tuple, Exception
+        Exception types to avoid logging if they fail
     """
-    if len(args) == 1 and isinstance(args[0], Iterable):
-        args = args[0]
     tasks = gen.WaitIterator(*args)
     results = [None for _ in args]
     while not tasks.done():
-        result = yield tasks.next()
+        try:
+            result = yield tasks.next()
+        except Exception:
+            @gen.coroutine
+            def quiet():
+                """ Watch unfinished tasks
+
+                Otherwise if they err they get logged in a way that is hard to
+                control.  They need some other task to watch them so that they
+                are not orphaned
+                """
+                for task in list(tasks._unfinished):
+                    try:
+                        yield task
+                    except quiet_exceptions:
+                        pass
+            quiet()
+            raise
+
         results[tasks.current_index] = result
     raise gen.Return(results)
 
@@ -210,19 +238,12 @@ def sync(loop, func, *args, **kwargs):
     Run coroutine in loop running in separate thread.
     """
     # Tornado's PollIOLoop doesn't raise when using closed, do it ourselves
-    if ((isinstance(loop, PollIOLoop) and getattr(loop, '_closing', False)) or
-        (hasattr(loop, 'asyncio_loop') and loop.asyncio_loop._closed)):
+    if PollIOLoop and ((isinstance(loop, PollIOLoop) and getattr(loop, '_closing', False)) or
+            (hasattr(loop, 'asyncio_loop') and loop.asyncio_loop._closed)):
         raise RuntimeError("IOLoop is closed")
 
     timeout = kwargs.pop('callback_timeout', None)
 
-    def make_coro():
-        coro = gen.maybe_future(func(*args, **kwargs))
-        if timeout is None:
-            return coro
-        else:
-            return gen.with_timeout(timedelta(seconds=timeout), coro)
-
     e = threading.Event()
     main_tid = get_thread_identity()
     result = [None]
@@ -235,7 +256,10 @@ def sync(loop, func, *args, **kwargs):
                 raise RuntimeError("sync() called from thread of running loop")
             yield gen.moment
             thread_state.asynchronous = True
-            result[0] = yield make_coro()
+            future = func(*args, **kwargs)
+            if timeout is not None:
+                future = gen.with_timeout(timedelta(seconds=timeout), future)
+            result[0] = yield future
         except Exception as exc:
             error[0] = sys.exc_info()
         finally:
@@ -651,7 +675,7 @@ def silence_logging(level, root='distributed'):
     return old
 
 
-@memoize
+@toolz.memoize
 def ensure_ip(hostname):
     """ Ensure that address is an IP address
 
@@ -776,22 +800,23 @@ def _maybe_complex(task):
             type(task) is dict and any(map(_maybe_complex, task.values())))
 
 
-def str_graph(dsk, extra_values=()):
-    def convert(task):
-        if type(task) is list:
-            return [convert(v) for v in task]
-        if type(task) is dict:
-            return valmap(convert, task)
-        if istask(task):
-            return (task[0],) + tuple(map(convert, task[1:]))
-        try:
-            if task in dsk or task in extra_values:
-                return tokey(task)
-        except TypeError:
-            pass
-        return task
+def convert(task, dsk, extra_values):
+    if type(task) is list:
+        return [convert(v, dsk, extra_values) for v in task]
+    if type(task) is dict:
+        return {k: convert(v, dsk, extra_values) for k, v in task.items()}
+    if istask(task):
+        return (task[0],) + tuple(convert(x, dsk, extra_values) for x in task[1:])
+    try:
+        if task in dsk or task in extra_values:
+            return tokey(task)
+    except TypeError:
+        pass
+    return task
 
-    return {tokey(k): convert(v) for k, v in dsk.items()}
+
+def str_graph(dsk, extra_values=()):
+    return {tokey(k): convert(v, dsk, extra_values) for k, v in dsk.items()}
 
 
 def seek_delimiter(file, delimiter, blocksize):
@@ -987,21 +1012,21 @@ def open_port(host=''):
 
 
 def import_file(path):
-    """ Loads modules for a file (.py, .pyc, .zip, .egg) """
+    """ Loads modules for a file (.py, .zip, .egg) """
     directory, filename = os.path.split(path)
     name, ext = os.path.splitext(filename)
     names_to_import = []
     tmp_python_path = None
 
-    if ext in ('.py', '.pyc'):
+    if ext in ('.py'):  # , '.pyc'):
         if directory not in sys.path:
             tmp_python_path = directory
         names_to_import.append(name)
-        # Ensures that no pyc file will be reused
+    if ext == '.py':  # Ensure that no pyc file will be reused
         cache_file = cache_from_source(path)
-        if os.path.exists(cache_file):
+        with ignoring(OSError):
             os.remove(cache_file)
-    if ext in ('.egg', '.zip'):
+    if ext in ('.egg', '.zip', '.pyz'):
         if path not in sys.path:
             sys.path.insert(0, path)
         if ext == '.egg':
@@ -1009,7 +1034,7 @@ def import_file(path):
             pkgs = pkg_resources.find_distributions(path)
             for pkg in pkgs:
                 names_to_import.append(pkg.project_name)
-        elif ext == '.zip':
+        elif ext in ('.zip', '.pyz'):
             names_to_import.append(name)
 
     loaded = []
@@ -1062,7 +1087,15 @@ def format_bytes(n):
     '12.35 MB'
     >>> format_bytes(1234567890)
     '1.23 GB'
+    >>> format_bytes(1234567890000)
+    '1.23 TB'
+    >>> format_bytes(1234567890000000)
+    '1.23 PB'
     """
+    if n > 1e15:
+        return '%0.2f PB' % (n / 1e15)
+    if n > 1e12:
+        return '%0.2f TB' % (n / 1e12)
     if n > 1e9:
         return '%0.2f GB' % (n / 1e9)
     if n > 1e6:
@@ -1238,7 +1271,10 @@ else:
         if isinstance(frame, _bytes_like):
             return len(frame)
         else:
-            return frame.nbytes
+            try:
+                return frame.nbytes
+            except AttributeError:
+                return len(frame)
 
 
 def PeriodicCallback(callback, callback_time, io_loop=None):
@@ -1358,3 +1394,27 @@ def reset_logger_locks():
 # Only bother if asyncio has been loaded by Tornado
 if 'asyncio' in sys.modules:
     fix_asyncio_event_loop_policy(sys.modules['asyncio'])
+
+
+def has_keyword(func, keyword):
+    if PY3:
+        return keyword in inspect.signature(func).parameters
+    else:
+        # https://stackoverflow.com/questions/50100498/determine-keywords-of-a-tornado-coroutine
+        if gen.is_coroutine_function(func):
+            func = func.__wrapped__
+        return keyword in inspect.getargspec(func).args
+
+
+# from bokeh.palettes import viridis
+# palette = viridis(18)
+palette = ['#440154', '#471669', '#472A79', '#433C84', '#3C4D8A', '#355D8C',
+           '#2E6C8E', '#287A8E', '#23898D', '#1E978A', '#20A585', '#2EB27C',
+           '#45BF6F', '#64CB5D', '#88D547', '#AFDC2E', '#D7E219', '#FDE724']
+
+
+@toolz.memoize
+def color_of(x, palette=palette):
+    h = md5(str(x).encode())
+    n = int(h.hexdigest()[:8], 16)
+    return palette[n % len(palette)]
