@@ -1,35 +1,27 @@
 from __future__ import print_function, division, absolute_import
 
-from collections import Iterable, defaultdict
+from collections import defaultdict
 from itertools import cycle
 import random
-import socket
-import uuid
 
 from tornado import gen
 from tornado.gen import Return
-from tornado.ioloop import IOLoop
-from tornado.iostream import StreamClosedError
 
-from dask.base import tokenize
-from toolz import merge, concat, groupby, drop, valmap
+from toolz import merge, concat, groupby, drop
 
-from .core import rpc, coerce_to_rpc, coerce_to_address
-from .utils import ignore_exceptions, All, log_errors, tokey, sync
-from .protocol.pickle import dumps, loads
-
-
-no_default = '__no_default__'
+from .core import rpc
+from .utils import All, tokey
 
 
 @gen.coroutine
-def gather_from_workers(who_has, rpc=rpc, close=True, permissive=False):
+def gather_from_workers(who_has, rpc, close=True, serializers=None, who=None):
     """ Gather data directly from peers
 
     Parameters
     ----------
     who_has: dict
         Dict mapping keys to sets of workers that may have that key
+    rpc: callable
 
     Returns dict mapping key to value
 
@@ -38,7 +30,10 @@ def gather_from_workers(who_has, rpc=rpc, close=True, permissive=False):
     gather
     _gather
     """
+    from .worker import get_data_from_worker
     bad_addresses = set()
+    missing_workers = set()
+    original_who_has = who_has
     who_has = {k: set(v) for k, v in who_has.items()}
     results = dict()
     all_bad_keys = set()
@@ -57,29 +52,32 @@ def gather_from_workers(who_has, rpc=rpc, close=True, permissive=False):
             except IndexError:
                 bad_keys.add(key)
         if bad_keys:
-            if permissive:
-                all_bad_keys |= bad_keys
-            else:
-                raise KeyError(*bad_keys)
+            all_bad_keys |= bad_keys
 
         rpcs = {addr: rpc(addr) for addr in d}
         try:
-            coroutines = [rpcs[address].get_data(keys=keys, close=close)
-                          for address, keys in d.items()]
-            response = yield ignore_exceptions(coroutines, socket.error,
-                                               StreamClosedError)
+            coroutines = {address: get_data_from_worker(rpc, keys, address,
+                                                        who=who,
+                                                        serializers=serializers,
+                                                        max_connections=False)
+                          for address, keys in d.items()}
+            response = {}
+            for worker, c in coroutines.items():
+                try:
+                    r = yield c
+                except EnvironmentError:
+                    missing_workers.add(worker)
+                else:
+                    response.update(r['data'])
         finally:
             for r in rpcs.values():
                 r.close_rpc()
 
-        response = merge(response)
         bad_addresses |= {v for k, v in rev.items() if k not in response}
-        results.update(merge(response))
+        results.update(response)
 
-    if permissive:
-        raise Return((results, all_bad_keys))
-    else:
-        raise Return(results)
+    bad_keys = {k: list(original_who_has[k]) for k in all_bad_keys}
+    raise Return((results, bad_keys, list(missing_workers)))
 
 
 class WrappedKey(object):
@@ -92,6 +90,7 @@ class WrappedKey(object):
     only be accessed in a certain way.  Schedulers may have particular needs
     that can only be addressed by additional metadata.
     """
+
     def __init__(self, key):
         self.key = key
 
@@ -100,7 +99,7 @@ _round_robin_counter = [0]
 
 
 @gen.coroutine
-def scatter_to_workers(ncores, data, report=True, serialize=True):
+def scatter_to_workers(ncores, data, rpc=rpc, report=True, serializers=None):
     """ Scatter data directly to workers
 
     This distributes data in a round-robin fashion to a set of workers based on
@@ -109,38 +108,29 @@ def scatter_to_workers(ncores, data, report=True, serialize=True):
 
     See scatter for parameter docstring
     """
-    if isinstance(ncores, Iterable) and not isinstance(ncores, dict):
-        k = len(data) // len(ncores)
-        ncores = {coerce_to_address(worker): k for worker in ncores}
+    assert isinstance(ncores, dict)
+    assert isinstance(data, dict)
 
     workers = list(concat([w] * nc for w, nc in ncores.items()))
-    if isinstance(data, dict):
-        names, data = list(zip(*data.items()))
-    else:
-        names = []
-        for x in data:
-            try:
-                names.append(tokenize(x))
-            except:
-                names.append(str(uuid.uuid1()))
+    names, data = list(zip(*data.items()))
 
     worker_iter = drop(_round_robin_counter[0] % len(workers), cycle(workers))
     _round_robin_counter[0] += len(data)
 
     L = list(zip(worker_iter, names, data))
     d = groupby(0, L)
-    d = {worker: {key: dumps(value) if serialize else value
-                   for _, key, value in v}
-          for worker, v in d.items()}
+    d = {worker: {key: value for _, key, value in v}
+         for worker, v in d.items()}
 
     rpcs = {addr: rpc(addr) for addr in d}
     try:
-        out = yield All([rpcs[address].update_data(data=v,
-                                                 close=True, report=report)
-                     for address, v in d.items()])
+        out = yield All([rpcs[address].update_data(data=v, report=report,
+                                                   serializers=serializers)
+                         for address, v in d.items()])
     finally:
         for r in rpcs.values():
             r.close_rpc()
+
     nbytes = merge(o['nbytes'] for o in out)
 
     who_has = {k: [w for w, _, _ in v] for k, v in groupby(1, L).items()}
@@ -206,7 +196,7 @@ def unpack_remotedata(o, byte_keys=False, myset=None):
         return o
 
 
-def pack_data(o, d):
+def pack_data(o, d, key_types=object):
     """ Merge known data into tuple or dict
 
     Parameters
@@ -228,19 +218,14 @@ def pack_data(o, d):
     """
     typ = type(o)
     try:
-        if o in d:
+        if isinstance(o, key_types) and o in d:
             return d[o]
     except TypeError:
         pass
 
     if typ in collection_types:
-        return typ([pack_data(x, d) for x in o])
+        return typ([pack_data(x, d, key_types=key_types) for x in o])
     elif typ is dict:
-        return {k: pack_data(v, d) for k, v in o.items()}
+        return {k: pack_data(v, d, key_types=key_types) for k, v in o.items()}
     else:
-        try:
-            if o in d:
-                return d[o]
-        except TypeError:
-            pass
         return o
